VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Exporter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ==== Excel export manager =========
Option Explicit

Private links_ As API_XLRecordsWrapper
Private shapes_ As API_XLRecordsWrapper
Private mentions_ As API_XLRecordsWrapper
Private cycles_ As API_XLRecordsWrapper
Private graph_ As CDS_Graph
Private terms_ As Scripting.Dictionary
Private mutualEdges_ As Scripting.Dictionary
Private thePage_ As Visio.Page

Public Function Init(iOut As Excel.Workbook)
  Set links_ = New API_XLRecordsWrapper: Call links_.Init(iOut.Sheets(XL_SHEET_LINKS))
  Set shapes_ = New API_XLRecordsWrapper: Call shapes_.Init(iOut.Sheets(XL_SHEET_SHAPES))
  Set mentions_ = New API_XLRecordsWrapper: Call mentions_.Init(iOut.Sheets(XL_SHEET_MENTIONS))
  Set cycles_ = New API_XLRecordsWrapper: Call cycles_.Init(iOut.Sheets(XL_SHEET_CYCLES), 1)
  Set graph_ = New CDS_Graph
  Set terms_ = New Scripting.Dictionary
  Set mutualEdges_ = New Scripting.Dictionary
End Function

Public Function ScanDataFrom(iSource As Visio.Page)
  Set thePage_ = iSource
  
  Call CSE_ProgressBar.Init(" ", sHeader:=" ", maxVal:=2 * thePage_.Shapes.Count)
  Call CSE_ProgressBar.ShowModeless
  
  Call ScanShapes
  Call ProcessDuplicates
  Call OutputCycles

  Unload CSE_ProgressBar
End Function

' ========
Private Function ScanShapes()
  Dim aShape As Visio.Shape
  For Each aShape In thePage_.Shapes
    If aShape.OneD Then
      Call ScanLink(aShape)
    Else
      Call ScanTerm(aShape)
    End If
    Call CSE_ProgressBar.IncrementA
  Next aShape
End Function

Private Function ProcessDuplicates()
  Dim aShape As Visio.Shape
  For Each aShape In thePage_.Shapes
    If aShape.OneD Or Not aShape.CellExistsU(VSO_CELL_SOLID, visExistsAnywhere) Then _
      GoTo NEXT_SHAPE
    If aShape.CellsU(VSO_CELL_SOLID) Then _
      GoTo NEXT_SHAPE
    
    Dim sText$
    If GetShapeType(aShape) = STYPE_DERIVED Then
      sText = aShape.Shapes(S_D_DEFINITION).Text
    Else
      sText = aShape.Text
    End If
    If Not terms_.Exists(sText) Then _
      GoTo NEXT_SHAPE
    
    Dim originalID&: originalID = terms_(sText)
    If originalID = aShape.ID Then _
      GoTo NEXT_SHAPE
      
    Dim edge1 As CDS_Edge: Set edge1 = graph_.AddEdge(aShape.ID, originalID)
    If Not edge1 Is Nothing Then _
      Call mutualEdges_.Add(edge1.ID, 0)
    Dim edge2 As CDS_Edge: Set edge2 = graph_.AddEdge(originalID, aShape.ID)
    If Not edge2 Is Nothing Then _
      Call mutualEdges_.Add(edge2.ID, 0)
    
NEXT_SHAPE:
    Call CSE_ProgressBar.IncrementA
  Next aShape
End Function

Private Function OutputCycles()
  Dim scc As New API_StrongComponents
  Dim components As Collection: Set components = scc.GetComponents(graph_)
  Dim vComponent As Scripting.Dictionary
  Dim nShapeID As Variant
  For Each vComponent In components
    If CheckCycle(vComponent) Then
      Dim nColumn&: nColumn = 1
      For Each nShapeID In vComponent
        Dim aShape As Visio.Shape: Set aShape = thePage_.Shapes.ItemFromID(nShapeID)
        If aShape.Shapes.Count = 3 Then
          cycles_.Field(nColumn) = aShape.Shapes(S_D_TERM).Text & " (" & nShapeID & ")"
        Else
          cycles_.Field(nColumn) = aShape.Text & " (" & nShapeID & ")"
        End If
        nColumn = nColumn + 1
      Next nShapeID
      Call cycles_.MoveNext
    End If
  Next vComponent
End Function

' ============
Private Function ScanLink(target As Visio.Shape)
  If target.Connects.Count <> 2 Then _
    Exit Function
    
  Dim nSource$: nSource = target.Connects(1).ToSheet.ID
  Dim nDest$: nDest = target.Connects(2).ToSheet.ID
  
  links_.Field(1) = target.ID
  links_.Field(2) = nSource
  links_.Field(3) = nDest
  
  If GetShapeType(target) = STYPE_LINK Then
    Dim tLink&: tLink = target.CellsU(VSO_CELL_LTYPE)
    links_.Field(4) = tLink
    links_.Field(5) = IIf(target.CellsU(VSO_CELL_SOLID), 0, 1)
    links_.Field(6) = IIf(target.CellsU(VSO_CELL_BOLD), 1, 0)
    
    If tLink <> LT_OMON And tLink <> LT_CONSISTS Then
      Call graph_.AddEdge(nSource, nDest)
      If tLink = LT_SYNON Then
        Dim newEdge As CDS_Edge: Set newEdge = graph_.AddEdge(nDest, nSource)
        If Not newEdge Is Nothing Then
          mutualEdges_.Item(newEdge.ID) = 0
          mutualEdges_.Item(newEdge.Reversed.ID) = 0
        End If
      End If
    End If
  End If
  
  Call links_.MoveNext
End Function

Private Function ScanTerm(target As Visio.Shape)
  Dim theType As ShapeType: theType = GetShapeType(target)
  If theType = STYPE_GRID Then _
    Exit Function
    
  Call graph_.AddNode(target.ID)
  
  shapes_.Field(XL_COL_ID) = target.ID
  shapes_.Field(XL_COL_TYPE) = theType
  shapes_.Field(XL_COL_PINX) = target.CellsU("PinX").result(visMillimeters)
  shapes_.Field(XL_COL_PINY) = target.CellsU("PinY").result(visMillimeters)
  shapes_.Field(XL_COL_WIDTH) = target.CellsU("Width").result(visMillimeters)
  shapes_.Field(XL_COL_HEIGHT) = target.CellsU("Height").result(visMillimeters)
  shapes_.Field(XL_COL_LINECOLOR) = target.CellsU("LineColor")
  shapes_.Field(XL_COL_TEXTCOLOR) = target.CellsU("Char.Color")
  
  If theType = STYPE_DERIVED Then
    shapes_.Field(XL_COL_TERMNAME) = target.Shapes(S_D_TERM).Text
    shapes_.Field(XL_COL_DEFINITION) = target.Shapes(S_D_DEFINITION).Text
    shapes_.Field(XL_COL_SOURCE) = target.Shapes(S_D_SOURCE).Text
    
    Call ScanMentions(target.ID, target.Shapes(S_D_DEFINITION))
  Else
    shapes_.Field(XL_COL_TERMNAME) = target.Text
  End If
  
  Dim parent As Visio.Shape: Set parent = GetContainingShape(target)
  If Not parent Is Nothing Then
    shapes_.Field(XL_COL_PARENT) = parent.ID
  End If
  
  If theType <= STYPE_DERIVED And theType >= STYPE_BASIC Then
    Dim isCopy&: isCopy = IIf(target.CellsU(VSO_CELL_SOLID), 0, 1)
    shapes_.Field(XL_COL_COPY) = isCopy
    If Not isCopy Then _
      Call terms_.Add(shapes_.Field(XL_COL_TERMNAME), target.ID)
  End If
  
  Call shapes_.MoveNext
End Function

Private Function ScanMentions(nID&, target As Visio.Shape)
  Dim sText$: sText = target.Text
  
  Dim nSection&: nSection = 0
  Dim nStart&: nStart = 0
  Dim nFinish&: nFinish = 1
  Dim nLast&: nLast = target.Characters.End
  
  Dim iChars As Visio.Characters: Set iChars = target.Characters
  iChars.Begin = nStart
  iChars.End = nFinish
  Do While nSection < target.RowCount(visSectionCharacter) - 1
    Do While nSection = iChars.CharPropsRow(visBiasRight) And nFinish <> nLast
      nFinish = nFinish + 1
      iChars.End = nFinish
    Loop
    nFinish = nFinish - 1

    Dim sColor$: sColor = target.CellsSRC(visSectionCharacter, nSection, visCharacterColor)
    Dim sStyle$: sStyle = target.CellsSRC(visSectionCharacter, nSection, visCharacterStyle)
    Dim sItem$: sItem = FixShapeText(VBA.Mid(sText, nStart + 1, nFinish - nStart))
    If IsNotEmpty(sItem) And (Not IsDefaultColor(sColor) Or Not IsDefaultStyle(sStyle)) Then
      mentions_.Field(XL_MENT_ID) = nID
      mentions_.Field(XL_MENT_TEXT) = sItem
      mentions_.Field(XL_MENT_START) = nStart
      mentions_.Field(XL_MENT_FINISH) = nFinish
      mentions_.Field(XL_MENT_COLOR) = sColor
      mentions_.Field(XL_MENT_STYLE) = sStyle
  
      Call mentions_.MoveNext
    End If
    
    iChars.Begin = nFinish
    nStart = nFinish
    nSection = nSection + 1
 Loop
End Function

Private Function FixShapeText(sText$) As String
  FixShapeText = Trim(sText)
  FixShapeText = Replace(FixShapeText, Chr(9), " ")
  FixShapeText = Replace(FixShapeText, Chr(160), " ")
  FixShapeText = Trim(FixShapeText)
End Function

Private Function IsNotEmpty(sText$) As Boolean
  IsNotEmpty = sText <> "" And Not sText Like "[,.!?:;]"
End Function

Private Function IsDefaultColor(sColor$) As Boolean
  IsDefaultColor = sColor = "0" Or sColor = "30"
End Function

Private Function IsDefaultStyle(sStyle$) As Boolean
  IsDefaultStyle = sStyle = "0"
End Function

Private Function CheckCycle(target As Scripting.Dictionary) As Boolean
  If target.Count < 2 Then
    CheckCycle = True
    Exit Function
  End If

  Dim anEdge As CDS_Edge
  For Each anEdge In graph_.FilterInternalEdges(target)
    If Not mutualEdges_.Exists(anEdge.ID) Then
      CheckCycle = True
      Exit Function
    End If
  Next anEdge
  CheckCycle = False
End Function

