Attribute VB_Name = "Main"
' =============      ==========
Option Explicit

Public Const PRODUCT_VERSION = "1.3.0"
Public Const PRODUCT_NAME = "Concept-Defs"

Public g_VersionTimer As Long

Public Sub StartVersionCheck()
  g_VersionTimer = SetTimer(0, 0, CP_VERSION_MSG_DELAY, AddressOf OnVersionCheck)
End Sub

Public Function OnVersionCheck(ByVal nHwnd As Long, ByVal uMsg As Long, ByVal nEvent As Long, ByVal nTime As Long)
  Call KillTimer(0, g_VersionTimer)
  Call VersionValidate(PRODUCT_NAME, PRODUCT_VERSION)
End Function

Public Sub FixOldShapes()
  Dim thePage As Visio.Page: Set thePage = ActivePage
  Dim vsoShape As Visio.Shape
  Dim i&: i = 1
  For i = 1 To thePage.Shapes.Count Step 1
    Set vsoShape = thePage.Shapes(i)
    If GetShapeType(vsoShape) = STYPE_DERIVED Then
      vsoShape.Shapes(2).CellsU("Width").Formula = "20mm"
      vsoShape.Shapes(3).CellsU("Width").FormulaU = "=Sheet." & CStr(vsoShape.ID) & "!Width - Sheet." & CStr(vsoShape.Shapes(2).ID) & "!Width"
    End If
  Next i
End Sub

Public Sub RunImportExcel()
  Dim theData As Collection: Set theData = PromptExcelData
  If theData Is Nothing Then _
    Exit Sub
  
  Dim vsoAppl As New API_VsoWrapper: Call vsoAppl.SetDocument(ThisDocument)
  Call vsoAppl.PauseUI
  Call GlobalUndo.BeginScope(" ")
  
  Call CSE_ProgressBar.Init(" ", sHeader:=" ...", maxVal:=theData.Count)
  Call CSE_ProgressBar.ShowModeless
  
  Dim baseCount&, termCount&
  Call BuildFromTerms(theData, baseCount, termCount)
  
  Call Unload(CSE_ProgressBar)
  
  Call GlobalUndo.EndScope
  Call vsoAppl.ResumeUI
  
  Call UserInteraction.ShowMessage(IM_GENEARION_COMPLETE, baseCount, termCount)
End Sub

Public Sub RunAddNotion()
  Call NewConcept.Show
  If NewConcept.isCancelled_ = True Then _
    Exit Sub
  If IsNumeric(NewConcept.RankInput.Value) = False Then
    Call UserInteraction.ShowMessage(EM_INVALID_LEVEL)
    Exit Sub
  End If
  
  Dim term As TermItem: Set term = NewConcept.GetTerm
  Dim iRank%: iRank = NewConcept.RankInput.Value
  Call Unload(NewConcept)
  
  Call GlobalUndo.BeginScope(" ")
  
  Call AddTermTo(term, ActivePage, iRank)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunGridPlus10()
  Call GlobalUndo.BeginScope(" ")
  
  If Not ActiveGrid.IncrementWidth(Visio.Application.ConvertResult(100, visMillimeters, visInches)) Then _
    Call UIShowMessage(EM_INVALID_WIDTH)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunGridMinus10()
  Call GlobalUndo.BeginScope(" ")
  
  If Not ActiveGrid.IncrementWidth(Visio.Application.ConvertResult(-100, visMillimeters, visInches)) Then _
    Call UIShowMessage(EM_INVALID_WIDTH)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunGridChange()
  Dim sDiff$: sDiff = UserInteraction.PromptInput("     ", " ", 100)
  If Not IsNumeric(sDiff) Then _
    Exit Sub
  Dim mmDiff&: mmDiff = CLng(sDiff)
  
  Call GlobalUndo.BeginScope(" ")
  
  If Not ActiveGrid.IncrementWidth(Visio.Application.ConvertResult(mmDiff, visMillimeters, visInches)) Then _
    Call UIShowMessage(EM_INVALID_WIDTH)

  Call GlobalUndo.EndScope
End Sub

Public Sub RunMoveUp()
Attribute RunMoveUp.VB_ProcData.VB_Invoke_Func = "q"
  Dim iSelected As Collection: Set iSelected = GetSelectedItems
  If iSelected.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_EMPTY_SELECTION)
    Exit Sub
  End If
  
  ThisDocument.DiagramServicesEnabled = visServiceStructureBasic
  Call GlobalUndo.BeginScope(" ")
  
  Dim rGrid As RankGrid: Set rGrid = ActiveGrid
  Dim vsoShape As Visio.Shape
  For Each vsoShape In iSelected
    Call rGrid.MoveUp(vsoShape)
  Next vsoShape
  
  Call GlobalUndo.EndScope
  ThisDocument.DiagramServicesEnabled = 0
End Sub

Public Sub RunMoveDown()
Attribute RunMoveDown.VB_ProcData.VB_Invoke_Func = "e"
  Dim iSelected As Collection: Set iSelected = GetSelectedItems
  If iSelected.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_EMPTY_SELECTION)
    Exit Sub
  End If
  
  ThisDocument.DiagramServicesEnabled = visServiceStructureBasic
  Call GlobalUndo.BeginScope(" ")
  
  Dim rGrid As RankGrid: Set rGrid = ActiveGrid
  Dim nOldSize%: nOldSize = rGrid.Size
  Dim vsoShape As Visio.Shape
  For Each vsoShape In iSelected
    Call rGrid.MoveDown(vsoShape)
  Next vsoShape
  
  If rGrid.Size <> nOldSize Then _
    Call VsoSelectShapes(iSelected, ActivePage)
  
  Call GlobalUndo.EndScope
  ThisDocument.DiagramServicesEnabled = 0
End Sub

Public Sub RunMoveToX()
Attribute RunMoveToX.VB_ProcData.VB_Invoke_Func = "w"
  Dim iSelected As Collection: Set iSelected = GetSelectedItems
  If iSelected.Count = 0 Then
    Call UserInteraction.ShowMessage(EM_EMPTY_SELECTION)
    Exit Sub
  End If
  
  Dim sRank$: sRank = UserInteraction.PromptInput("  ", "  ", 1)
  If Not IsNumeric(sRank) Then _
    Exit Sub
  Dim iRank%: iRank = VBA.Int(sRank)
  If iRank > RANK_MAX Or iRank < 0 Then
    Call UserInteraction.ShowMessage(EM_INVALID_LEVEL)
    Exit Sub
  End If
  
  ThisDocument.DiagramServicesEnabled = visServiceStructureBasic
  Call GlobalUndo.BeginScope("")
  
  Dim rGrid As RankGrid: Set rGrid = ActiveGrid
  Dim nOldSize%: nOldSize = rGrid.Size
  Dim vsoShape As Visio.Shape
  For Each vsoShape In iSelected
    Call rGrid.MoveToRank(vsoShape, iRank)
  Next vsoShape
  
  If rGrid.Size <> nOldSize Then _
    Call VsoSelectShapes(iSelected, ActivePage)
  
  Call GlobalUndo.EndScope
  ThisDocument.DiagramServicesEnabled = 0
End Sub

Public Sub RunExportExcel()
  Call Export2ExcelFrom(ActivePage)
  Call UserInteraction.ShowMessage(IM_EXPORT_XL_COMPLETE)
End Sub

Public Sub RunCompare()
  Dim sFileName$: sFileName = UserInteraction.PromptFileFilter( _
                                sInitialPath:=ThisDocument.Path, _
                                sDescription:="Visio file", _
                                sFilter:="*.vsd*", _
                                bNewApplication:=True)
  If sFileName = vbNullString Then _
    Exit Sub
  
  Dim thisDoc As Visio.Document: Set thisDoc = ActiveDocument
  Dim wrap As New API_VsoWrapper
  Call wrap.CreateApplication
  If wrap.OpenDocument(sFileName, nOpenFlags:=visOpenRO + visOpenMacrosDisabled) Is Nothing Then _
    Exit Sub
  
  Dim res As Boolean: res = ExportComparison(thisDoc, wrap.Document)
  Call wrap.ReleaseDocument
  
  If res Then _
    Call UserInteraction.ShowMessage(IM_EXPORT_DIFF_COMPLETE)
End Sub

Public Sub RunCleanUp()
  Call GlobalUndo.BeginScope("")
  
  Call ClearAllExceptGrid(Visio.ActivePage)
  Visio.ActiveWindow.ViewFit = visFitPage
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunAddGrid()
  Call GridDlg.Show
  If GridDlg.isCancelled_ = True Then _
    Exit Sub
  
  Dim nCount%: nCount = GridDlg.LevelCount
  Dim dTopMargin As Double: dTopMargin = Visio.Application.ConvertResult(GridDlg.FirstGap, visMillimeters, visInches)
  Dim dHeight As Double: dHeight = Visio.Application.ConvertResult(GridDlg.LevelHeight, visMillimeters, visInches)
  Dim dLeftMargin As Double: dLeftMargin = Visio.Application.ConvertResult(GRID_X_MARGIN, visMillimeters, visInches)
  Dim dWidth As Double: dWidth = ActivePage.PageSheet.CellsU("PageWidth") - dLeftMargin
  Unload GridDlg
  
  Call GlobalUndo.BeginScope(" ")
  
  Call ActiveGrid.CreateGrid(nCount, dHeight:=dHeight, dWidth:=dWidth, dTopMargin:=dTopMargin, dLeftMargin:=dLeftMargin)

  Call GlobalUndo.EndScope
  
End Sub

Public Sub RunDeleteGrid()
  Call GlobalUndo.BeginScope(" ")
  
  Call ActiveGrid.DeleteGrid
  
  Call GlobalUndo.EndScope
End Sub

Public Sub RunHelp()
  Dim objWord As Object: Set objWord = CreateObject("Word.Application")
  Dim objDoc As Object: Set objDoc = objWord.Documents.Open(APP_HELP, ReadOnly:=True)
  With objWord
    .Visible = True
    .ActiveWindow.View.ReadingLayout = False
    Call .Activate
  End With
End Sub

' ===== Context menu API ========
Public Function Automarkdown(target As Visio.Shape)
  Dim iPage As Visio.Page: Set iPage = target.parent
  Dim iTerms As Scripting.Dictionary: Set iTerms = ScanAllTerms(iPage)
  Dim iDefinition As Visio.Shape: Set iDefinition = target.Shapes(S_D_DEFINITION)
  
  Call GlobalUndo.BeginScope("")
  
  Dim sText$: sText = VBA.LCase(iDefinition.Text)
  
  Dim sTerm As Variant
  Dim iFoundRange As Variant
  Dim nStart&, nEnd&
  Dim iSource As Visio.Shape
  For Each sTerm In iTerms.Keys
    On Error GoTo SAFE_EXIT
    iFoundRange = AccessPython.CallFunction(PY_MODULE_TEXT, "find_substr", Array(sText, CStr(sTerm)))
    If VarType(iFoundRange) = vbString Then
      Call UserInteraction.ShowMessage(EM_PYTHON_ERROR, iFoundRange)
      GoTo SAFE_EXIT
    End If
    On Error GoTo 0
    nStart = iFoundRange(0)
    nEnd = iFoundRange(1)
    If nStart = nEnd Then _
      GoTo NEXT_TERM
    
    Set iSource = iTerms(sTerm)
    If iSource.ID = target.ID Then
      Call VsoApplyColorTo(iDefinition, nStart, nEnd, COLOR_INVALID)
    Else
      Call VsoApplyColorTo(iDefinition, nStart, nEnd, COLOR_MENTION)
      If Not VsoIsConnected(iSource, target) Then _
        Call iSource.AutoConnect(target, visAutoConnectDirNone)
    End If
    
NEXT_TERM:
  Next sTerm
  
SAFE_EXIT:
  Call GlobalUndo.EndScope
End Function

' =======
Private Function GetSelectedItems() As Collection ' of Visio.Shape
  Dim iShapes As New Collection
  
  Dim iSelected As Visio.Selection: Set iSelected = ActiveWindow.Selection
  Dim vsoShape As Visio.Shape
  For Each vsoShape In iSelected
    If Not vsoShape.OneD Then _
      If GetShapeType(vsoShape) <> STYPE_GRID Then _
        Call iShapes.Add(vsoShape)
NEXT_SHAPE:
  Next vsoShape
  
  Set GetSelectedItems = iShapes
End Function

Private Function ActiveGrid() As RankGrid
  Set ActiveGrid = New RankGrid
  Call ActiveGrid.Init(ActivePage)
End Function
