Attribute VB_Name = "MainImpl"
Option Explicit

Public Function ClearAllExceptGrid(target As Visio.Page)
  Dim vsoShape As Visio.Shape
  Dim i&: i = 1
  Do While i <= target.Shapes.Count
    Set vsoShape = target.Shapes(i)
    If GetShapeType(vsoShape) = STYPE_GRID Then
      i = i + 1
      GoTo Continue
    End If
    
    Call vsoShape.Delete
    i = 1
Continue:
  Loop
End Function

Public Function CreateItemShape(term As TermItem, target As Visio.Page) As Visio.Shape
  Dim res As Visio.Shape
  If term.Basic Then
    Set res = CreateShape(MASTER_BASIC, target)
    res.Characters.Text = term.name_
  Else
    Set res = CreateShape(MASTER_DERIVED, target)
    res.Shapes(S_D_TERM).Text = term.name_
    res.Shapes(S_D_SOURCE).Text = term.source_
    res.Shapes(S_D_DEFINITION).Text = term.definition_
    
    If InStr(term.definition_, Chr(13)) = 0 Then
      res.Shapes(S_D_DEFINITION).CellsU("Width").FormulaU = "=MAX(50mm, TEXTWIDTH(TheText)*Char.Size*1.4/Height + 0.2in)"
      res.CellsU("Width") = res.Shapes(S_D_SOURCE).CellsU("Width") + res.Shapes(S_D_DEFINITION).CellsU("Width")
      res.Shapes(S_D_DEFINITION).CellsU("Width").FormulaU = "=Sheet." & CStr(res.ID) & "!Width - Sheet." & CStr(res.Shapes(S_D_SOURCE).ID) & "!Width"
    End If
  End If
  Set CreateItemShape = res
End Function

Public Function CreateShape(sMaster$, iDestination As Visio.Page) As Visio.Shape
  Set CreateShape = iDestination.Drop(FindMaster(iDestination.Document, sMaster), 0, 0)
End Function

Public Function AddTermTo(target As TermItem, iDestination As Visio.Page, iRank%)
  Dim itemShape As Visio.Shape: Set itemShape = CreateItemShape(target, iDestination)
  Dim rGrid As New RankGrid: Call rGrid.Init(iDestination)
  
  Dim baseX As Double: baseX = GetMostRightPos
  If baseX = MOST_LEFT Then
    baseX = iDestination.Shapes("SLevel1").CellsU("pinx") - iDestination.Shapes("SLevel1").CellsU("width") / 2#
  End If
  itemShape.CellsU("PinX") = baseX + itemShape.CellsU("Width") / 2#
  
  Call rGrid.MoveToRank(itemShape, iRank)
  Call CenterScreenOnShape(itemShape)
End Function

Public Function PromptExcelData() As Collection
  Dim sFileName$: sFileName = UserInteraction.PromptFileFilter( _
                                sInitialPath:=ThisDocument.Path, _
                                sDescription:="Excel file", _
                                sFilter:="*.xls*;*.csv", _
                                bNewApplication:=True)
  If sFileName = "" Then _
    Exit Function
    
  Dim wrap As New API_XLWrapper
  If wrap.OpenDocument(sFileName) Is Nothing Then _
    Exit Function
  
  Dim theData As Collection
  Set theData = ScanExcel(wrap.Document.Worksheets(1))
  If theData Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_FILE_FORMAT)
    Exit Function
  End If
  
  Call wrap.ReleaseDocument
  Set PromptExcelData = theData
End Function

Public Function Export2ExcelFrom(target As Visio.Page)
  Dim templatePath$: templatePath = GetExportTemplate(XL_TEMPLATE_NAME)
  Dim wrapper As New API_XLWrapper
  Dim wb As Excel.Workbook: Set wb = wrapper.NewDocument(templatePath)
  If wb Is Nothing Then _
    Exit Function
    
  If templatePath = vbNullString Then
    Call UserInteraction.ShowMessage(EM_TEMPLATE_MISSING)
    Call wb.Sheets.Add
    Call wb.Sheets.Add
    Call wb.Sheets.Add
    wb.Sheets(1).Name = XL_SHEET_SHAPES
    wb.Sheets(2).Name = XL_SHEET_LINKS
    wb.Sheets(3).Name = XL_SHEET_MENTIONS
    wb.Sheets(4).Name = XL_SHEET_CYCLES
  End If
    
  Call wrapper.PauseUI
  
  Dim helper As New Exporter:  Call helper.Init(wb)
  Call helper.ScanDataFrom(target)
  
  Call wrapper.ResumeUI
  Call Unload(CSE_ProgressBar)
End Function
  
Public Function ExportComparison(iSource As Visio.Document, target As Visio.Document) As Boolean
  ExportComparison = False
  
  Dim xlAppl As New API_XLWrapper
  Dim outWB As Excel.Workbook: Set outWB = xlAppl.NewDocument
  If outWB Is Nothing Then _
    Exit Function
    
  Call xlAppl.PauseUI
  
  Call PrepareCompareOutput(outWB)
  Dim Exporter As New ComparatorExporter: Call Exporter.Init(outWB)
  Call Exporter.RunCompare(iSource.Pages(1), target.Pages(1))
  Call FinalizeCompare(outWB)
  
  Call xlAppl.ResumeUI
  
  ExportComparison = True
End Function

Public Function BuildFromTerms(theData As Collection, ByRef baseCount&, ByRef termCount&)
  Dim resPage As Visio.Page: Set resPage = ActivePage
  
  Dim basicWidth As Double: basicWidth = resPage.Document.Masters(MASTER_BASIC).Shapes(1).CellsU("Width")
  Dim basicHeight As Double: basicHeight = resPage.Document.Masters(MASTER_BASIC).Shapes(1).CellsU("Height")
  Dim derivedWidth As Double: derivedWidth = resPage.Document.Masters(MASTER_DERIVED).Shapes(1).Shapes(SHAPE_DERVIED).CellsU("Width")
  Dim derivedHeight As Double: derivedHeight = resPage.Document.Masters(MASTER_DERIVED).Shapes(1).CellsU("Height")
  
  Dim basicX As Double: basicX = resPage.PageSheet.CellsU("XRulerOrigin") - BASE_START
  Dim basicY As Double: basicY = resPage.PageSheet.CellsU("YRulerOrigin") - basicHeight / 2#
  Dim derivedX As Double: derivedX = resPage.PageSheet.CellsU("XRulerOrigin")
  Dim derivedY As Double: derivedY = resPage.PageSheet.CellsU("YRulerOrigin") + V_GAP
  
  Dim term As TermItem
  Dim itemShape As Visio.Shape
  Dim nItem&: nItem = 1: baseCount = 0: termCount = 0
  Do While nItem <= theData.Count
    Set term = theData.Item(nItem)
    Set itemShape = CreateItemShape(term, resPage)
    If itemShape Is Nothing Then _
      GoTo NEXT_ITEM
      
    With itemShape
      If term.Basic = True Then
        itemShape.CellsU("PinX") = basicX - (BASIC_COLOMNS - 1 - baseCount Mod BASIC_COLOMNS) * (H_GAP + basicWidth) + itemShape.CellsU("Width") / 2#
        itemShape.CellsU("PinY") = basicY - (baseCount \ BASIC_COLOMNS) * (V_GAP + basicHeight)
        baseCount = baseCount + 1
      Else
        itemShape.CellsU("PinX") = derivedX + (termCount \ DERIVED_ROWS) * (H_GAP + derivedWidth) + itemShape.CellsU("Width") / 2#
        itemShape.CellsU("PinY") = derivedY + (1 + (termCount Mod DERIVED_ROWS)) * (V_GAP + derivedHeight)
        termCount = termCount + 1
      End If
    End With
    
NEXT_ITEM:
    nItem = nItem + 1
    Call CSE_ProgressBar.IncrementA
  Loop
End Function

Public Function ScanAllTerms(target As Visio.Page) As Scripting.Dictionary
  Dim iData As New Scripting.Dictionary
  
  Dim theType As ShapeType
  Dim aShape As Visio.Shape
  For Each aShape In target.Shapes
    theType = GetShapeType(aShape)
    If theType = STYPE_GRID Or theType = STYPE_UNKNOWN Then _
      GoTo NEXT_SHAPE
    If IsShapeDuplicate(aShape) Then _
      GoTo NEXT_SHAPE
    
    Dim sText$
    If theType = STYPE_DERIVED Then
      sText = aShape.Shapes(S_D_TERM).Text
    Else
      sText = aShape.Text
    End If
    If sText <> "" Then _
      sText = VBA.LCase(VBA.Left(sText, 1)) & VBA.Right(sText, VBA.Len(sText) - 1)
    
    If Not iData.Exists(sText) Then _
      Set iData(sText) = aShape

NEXT_SHAPE:
  Next aShape
  
  Set ScanAllTerms = iData
End Function

' ==============
Private Function CenterScreenOnShape(target As Visio.Shape)
  Dim pinLeft As Double, pinTop As Double, pinWidth As Double, pinHeight As Double
  Call Visio.Application.ActiveWindow.GetViewRect(pinLeft, pinTop, pinWidth, pinHeight)
  Call Visio.Application.ActiveWindow.SetViewRect(target.CellsU("PinX") - pinWidth / 2#, _
                          target.CellsU("PinY") + pinHeight / 2#, _
                          pinWidth, pinHeight)
End Function


Private Function ScanExcel(dataSheet As Excel.Worksheet) As Collection
' Extracting data from Excel
  Dim result As New Collection
    
  Dim lstrow&:  lstrow = dataSheet.Cells(dataSheet.Rows.Count, 1).End(xlUp).Row
  Dim nRow&: nRow = 2
  Dim newItem As TermItem
  For nRow = 2 To lstrow
    Set newItem = New TermItem
    With newItem
      .name_ = dataSheet.Cells(nRow, 1)
      .definition_ = dataSheet.Cells(nRow, 2)
      .source_ = dataSheet.Cells(nRow, 3)
    End With
    
    Call result.Add(newItem)
  Next nRow
    
  Set ScanExcel = result
End Function

Private Function PrepareCompareOutput(target As Excel.Workbook)
  Call target.Sheets.Add
  Call target.Sheets.Add
  With target.Sheets(1)
    .Name = "Source1"
    .Cells(1, XL_TS_ID) = "ShapeID"
    .Cells(1, XL_TS_NAME) = ""
    .Cells(1, XL_TS_DEFINITION) = ""
    .Columns(XL_TS_NAME).ColumnWidth = 30
    .Columns(XL_TS_DEFINITION).ColumnWidth = 60
    .Rows(1).Font.Bold = True
  End With
  
  With target.Sheets(2)
    .Name = "Source2"
    .Cells(1, XL_TS_ID) = "ShapeID"
    .Cells(1, XL_TS_NAME) = ""
    .Cells(1, XL_TS_DEFINITION) = ""
    .Columns(XL_TS_NAME).ColumnWidth = 30
    .Columns(XL_TS_DEFINITION).ColumnWidth = 60
    .Rows(1).Font.Bold = True
  End With
  
  With target.Sheets(3)
    .Name = "Compare"
    .Cells(1, XL_COMP_1ST) = " 1"
    .Cells(1, XL_COMP_2ND) = " 2"
    .Cells(1, XL_COMP_SHARED) = ""
    .Columns(XL_COMP_1ST).ColumnWidth = 30
    .Columns(XL_COMP_2ND).ColumnWidth = 30
    .Columns(XL_COMP_SHARED).ColumnWidth = 30
    .Rows(1).Font.Bold = True
  End With
  
  Dim ws As Excel.Worksheet
  For Each ws In target.Worksheets
    Call ws.Activate
    With Excel.ActiveWindow
      .SplitColumn = 0
      .SplitRow = 1
      .FreezePanes = True
    End With
  Next ws
End Function

Private Function FinalizeCompare(target As Excel.Workbook)
  Call target.Sheets(1).Range("A:C").Sort(key1:=target.Sheets(1).Range("B1"), order1:=xlAscending, Header:=xlYes)
  Call target.Sheets(2).Range("A:C").Sort(key1:=target.Sheets(2).Range("B1"), order1:=xlAscending, Header:=xlYes)
  Call target.Sheets(3).Range("A:A").Sort(key1:=target.Sheets(3).Range("A1"), order1:=xlAscending, Header:=xlYes)
  Call target.Sheets(3).Range("B:B").Sort(key1:=target.Sheets(3).Range("B1"), order1:=xlAscending, Header:=xlYes)
  Call target.Sheets(3).Range("C:C").Sort(key1:=target.Sheets(3).Range("C1"), order1:=xlAscending, Header:=xlYes)
End Function

Private Function GetMostRightPos() As Double
  GetMostRightPos = MOST_LEFT
  
  Dim aShape As Visio.Shape
  Dim rightX As Double
  For Each aShape In ActivePage.Shapes
    If GetShapeType(aShape) = STYPE_GRID Then _
      GoTo NEXT_SHAPE
    rightX = aShape.CellsU("PinX") + aShape.CellsU("Width") / 2# + H_GAP
    GetMostRightPos = IIf(rightX <= GetMostRightPos, GetMostRightPos, rightX)
     
NEXT_SHAPE:
  Next aShape
End Function
